#if defined(UNICODE) && !defined(_UNICODE)
    #define _UNICODE
#elif defined(_UNICODE) && !defined(UNICODE)
    #define UNICODE
#endif

#include <tchar.h>
#include <windows.h>
#include <Freeimage.h>
#include "FreeImage001.h"

/*  Declare Windows procedure  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
FIBITMAP* GenericLoader(const char* lpszPathName, int flag);
RECT AdaptaArea(HWND, BITMAPINFOHEADER*);

/*  Make the class name into a global variable  */
TCHAR szClassName[ ] = _T("CodeBlocksWindowsApp");

int WINAPI WinMain (HINSTANCE hThisInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR lpszArgument,
                     int nCmdShow)
{
    HWND hwnd;               /* This is the handle for our window */
    MSG mensaje;            /* Here messages to the application are saved */
    WNDCLASSEX wincl;        /* Data structure for the windowclass */

    /* The Window structure */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = CS_DBLCLKS;                 /* Catch double-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon (hThisInstance, "Icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "Icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";               /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use Windows's default colour as the background of the window */
    wincl.hbrBackground = (HBRUSH) COLOR_BACKGROUND;

    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (
           0,                   /* Extended possibilites for variation */
           szClassName,         /* Classname */
           _T("FreeImage ejemplo 1"),       /* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           544,                 /* The programs width */
           375,                 /* and height in pixels */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           hThisInstance,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    /* Make the window visible on the screen */
    ShowWindow (hwnd, nCmdShow);

    /* Run the message loop. It will run until GetMessage() returns 0 */
    while(TRUE == GetMessage(&mensaje, NULL, 0, 0))
    {
        /* Translate virtual-key messages into character messages */
        TranslateMessage(&mensaje);
        /* Send message to WindowProcedure */
        DispatchMessage(&mensaje);
    }

    /* The program return-value is 0 - The value that PostQuitMessage() gave */
    return mensaje.wParam;
}


/*  This function is called by the Windows function DispatchMessage()  */

LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static FIBITMAP *dib = NULL;
    static HBITMAP bitmap;
    RECT re;
    HDC hdc;
    PAINTSTRUCT ps;
    HDC memDC;

    switch (message)                  /* handle the messages */
    {
        case WM_CREATE:
            dib = GenericLoader("meninas2.gif", GIF_PLAYBACK);
            break;
        case WM_PAINT:
            // Si no se pudo cargar el mapa de bits, retornar sin mostrar nada.
            if(!dib) return 0;
            re = AdaptaArea(hwnd, FreeImage_GetInfoHeader(dib));
            MoveWindow(hwnd, re.left, re.top, re.right, re.bottom, true);
            hdc = BeginPaint(hwnd, LPPAINTSTRUCT(&ps));
            bitmap = CreateDIBitmap(hdc, FreeImage_GetInfoHeader(dib),
                CBM_INIT, FreeImage_GetBits(dib), FreeImage_GetInfo(dib), DIB_RGB_COLORS);
            memDC = CreateCompatibleDC(hdc);
            GetClientRect(hwnd, (LPRECT)&re);
            SelectObject(memDC, bitmap);
            BitBlt(hdc, re.left, re.top, re.right-re.left, re.bottom-re.top, memDC, 0, 0, SRCCOPY);
            DeleteDC(memDC);
            EndPaint(hwnd, LPPAINTSTRUCT(&ps));
            return 0;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case CM_CARGABMP:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("meninas24.bmp", 0);
                    break;
                case CM_JPGDEFECTO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Baco.jpg", JPEG_DEFAULT);
                    break;
                case CM_JPGRAPIDO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Baco.jpg", JPEG_FAST);
                    break;
                case CM_JPGPRECISO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Baco.jpg", JPEG_ACCURATE);
                    break;
                case CM_JPGCMYK:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Baco.jpg", JPEG_CMYK);
                    break;
                case CM_JPGBLANCONEGRO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Baco.jpg", JPEG_GREYSCALE);
                    break;
                case CM_GIFDEFECTO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("meninas2.gif", GIF_DEFAULT);
                    break;
                case CM_PNGDEFECTO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Breda.png", PNG_DEFAULT);
                    break;
                case CM_PNGIGNORAGAMMA:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("Breda.png", PNG_IGNOREGAMMA);
                    break;
                case CM_ICODEFECTO:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("opcion.ico", ICO_DEFAULT);
                    break;
                case CM_ICOALFA:
                    FreeImage_Unload(dib);
                    dib = GenericLoader("opcion.ico", ICO_MAKEALPHA);
                    break;
            }
            InvalidateRect(hwnd, NULL, TRUE);
            break;
        case WM_DESTROY:
            FreeImage_Unload(dib);
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;
        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return 0;
}

FIBITMAP* GenericLoader(const char* lpszPathName, int flag)
{
    FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;

    // Verifica la firma y deduce el formato
    // (El segundo argumento no se usa an por FreeImage)
    fif = FreeImage_GetFileType(lpszPathName, 0);
    if(fif == FIF_UNKNOWN) {
        // no hay firma?
        // Intenta obtener el formato a partir de la extensin de fichero
        fif = FreeImage_GetFIFFromFilename(lpszPathName);
    }
    // Verifica si el plugin puede leer este tipo de mapa de bits...
    if((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
        // Lo cargamos
        FIBITMAP *dib = FreeImage_Load(fif, lpszPathName, flag);
        // unless a bad file format, we are done !
        return dib;
    } else
        return 0;
}

RECT AdaptaArea(HWND hwnd, BITMAPINFOHEADER *bmheader) {
    RECT Rect;
    RECT reWnd;
    RECT reClient;

    // Obtenemos las coordenadas y dimensiones de la ventana:
    GetWindowRect(hwnd, &reWnd);
    // Obtenemos las coordenadas y dimensiones del rea de cliente:
    GetClientRect(hwnd, &reClient);
    // Calcular las nuevas coordenadas y dimensiones de la ventana, de modo que
    // el rea de cliente tenga las mismas dimensiones que el mapa de bits:
    Rect.left = reWnd.left;
    Rect.top = reWnd.top;
    Rect.right = bmheader->biWidth+(reWnd.right-reWnd.left)-(reClient.right-reClient.left);
    Rect.bottom = bmheader->biHeight+(reWnd.bottom-reWnd.top)-(reClient.bottom-reClient.top);
    return Rect;
}
